from bolinette.controllers.file import ns as file_controller
from bolinette.controllers.user import ns as user_controller
from bolinette.controllers.role import ns as role_namespace
from bolinette.controllers.timezone import ns as tz_namespace
